
# q-expansion of newform 2550.2.d.l, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2550
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[851, [1, 0]], [1327, [-1, 0]], [751, [1, 0]]]
aps_data = [[0, -1], [0, 1], [0, 0], [0, 3], [-5, 0], [0, -4], [0, -1], [1, 0], [0, 4], [4, 0], [-1, 0], [0, -9], [-10, 0], [0, -11], [0, 9], [0, -3], [8, 0], [-14, 0], [0, -7], [14, 0], [0, -2], [5, 0], [0, -8], [2, 0], [0, -14], [9, 0], [0, -14], [0, -15], [-1, 0], [0, 3], [0, -2], [4, 0], [0, -6], [-8, 0], [-14, 0], [-24, 0], [0, -24], [0, 18], [0, 18], [0, 14], [12, 0], [-5, 0], [-9, 0], [0, -6], [0, -12], [1, 0], [-12, 0], [0, -6], [0, -21], [12, 0], [0, 6], [-5, 0], [-8, 0], [-18, 0], [0, -12], [0, 7], [-28, 0], [4, 0], [0, -13], [-18, 0], [0, -22], [0, 18], [0, 8], [10, 0], [0, 16], [0, 4], [5, 0], [0, 24], [0, 11], [10, 0], [0, -24], [25, 0], [0, 25], [0, -6], [4, 0], [0, 12], [-21, 0], [0, 15], [-30, 0], [26, 0], [-28, 0], [24, 0], [14, 0], [0, -27], [-20, 0], [0, 38], [41, 0], [0, 37], [-5, 0], [0, -36], [0, -6], [-30, 0], [0, -8], [-30, 0], [38, 0], [0, -34], [9, 0], [15, 0], [0, 20], [23, 0], [0, 2], [0, 9], [0, -16], [-14, 0], [-26, 0], [0, -45], [0, 6], [0, 0], [-39, 0], [-8, 0], [0, 32], [0, 10], [0, 19], [40, 0], [20, 0], [18, 0], [0, -44], [0, 8], [0, 10], [-12, 0], [-14, 0], [0, -4], [0, 18], [0, 8], [6, 0], [22, 0], [-49, 0], [18, 0], [0, 32], [0, 2], [17, 0], [0, -38], [-4, 0], [0, -46], [0, 0], [-11, 0], [0, -22], [0, 50], [0, -39], [-39, 0], [-22, 0], [-30, 0], [0, -8], [0, -39], [24, 0], [-10, 0], [0, -9], [0, -5], [25, 0], [0, -17], [0, 46], [-35, 0], [0, -48], [0, -12], [0, 24], [36, 0], [14, 0], [-1, 0], [0, -38], [18, 0], [0, -29], [0, -12], [0, -24], [-10, 0], [0, -48], [0, 28], [16, 0], [0, -25], [-42, 0], [0, 58], [-24, 0], [6, 0], [64, 0], [0, 26], [24, 0], [33, 0], [2, 0], [-50, 0], [0, 26], [-2, 0], [0, 18], [17, 0], [0, 55], [0, 24], [0, -31], [-39, 0], [0, -11], [0, 64], [-16, 0], [-52, 0], [0, -20], [0, -36], [-60, 0], [-34, 0], [0, -3], [0, -43], [28, 0], [0, -62], [0, 43], [0, -55], [42, 0], [-25, 0], [0, 32], [-58, 0], [8, 0], [0, -27], [50, 0], [0, 2], [59, 0], [-61, 0], [0, -38], [45, 0], [0, -49], [0, 64], [4, 0], [2, 0], [0, 12], [42, 0], [0, -46], [0, -49], [10, 0], [31, 0], [10, 0], [0, 7], [0, 32], [-58, 0], [0, -33], [-18, 0], [0, -32], [9, 0], [0, -46], [28, 0], [30, 0], [22, 0], [0, 51], [0, 7], [-70, 0], [0, -48], [-12, 0], [-63, 0], [0, -56], [3, 0], [0, 28], [70, 0], [0, 15], [6, 0], [0, -56], [-13, 0], [-19, 0], [0, 45], [0, -10], [-23, 0], [0, -48], [-68, 0], [0, 34], [34, 0], [31, 0], [0, -46], [0, -12], [0, 63], [0, 21], [0, 14], [-7, 0], [0, 58], [0, 69], [-64, 0], [-10, 0], [-20, 0], [0, 22], [0, 33], [58, 0], [0, 59], [0, -1], [-34, 0], [0, 5], [0, -28], [0, 56], [40, 0], [-79, 0], [50, 0], [0, 9], [61, 0], [0, -28], [38, 0], [0, -46], [21, 0], [0, -62], [0, -62], [58, 0], [66, 0], [-12, 0], [0, 36], [0, 66], [-1, 0], [0, -11], [52, 0], [-24, 0], [0, 10], [-35, 0], [0, 23], [0, -61], [0, 47], [55, 0]]
